/////////////////////////////////////////////////////////////////////////////
//                                                                            
// WWW                 WW                    WW                               
//  WW                 WW                    WW                               
//  WW                 WW                    WW                               
//  WW    WWW    WWW   WW  WW  WW WW  WWWW   WWWW                             
//  WW   WW WW  WW WW  WW WW   WW WW  WW WW  WW WW                            
//  WW   WW WW  WW WW  WWWW    WW WW  WW WW  WW WW                            
//  WW   WW WW  WW WW  WW WW   WW WW  WW WW  WW WW                            
// WWWW   WWW    WWW   WW  WW   WWWW  WWWW   WW WW                            
//                                    WW                                      
//                                    WW                                      
//
// Implements lookup table for sin and cos.
// Kevin Beason <beason@cs.fsu.edu>
// 10/24/02

#ifndef LOOKUP_H
#define LOOKUP_H 1

#include<math.h>
#include<stdlib.h>
#include<stdio.h>

#include "grid.h"
extern GridStruct Grid;

class LookupTable {
private:
  static const int DIM=1024;
  static const int DIM4=DIM/4;
  static const double pi2 = 2.0*M_PI;
  static const double npi2 = -2.0*M_PI;
  static const int DIMB = 8192;
  static const double BESSEL_MAX = 24.0;
  static const double BESSEL_MAX2 = 2.0*BESSEL_MAX;
  static const int DIME = 2048;
  static const double EXP_MAX = 5.0;
  double SinT[DIM];
  double BesT[DIMB];
  double ExpT[DIME];
  double *DisT;
  
public:
  void calcTable();
  void test();
  
  inline double lsin(double x){
    return (SinT[int(x/pi2 * DIM)%DIM]);
  }
  
  inline double lnsin(double x){
    return (-SinT[int(x/npi2 * DIM)%DIM]);
  }

  inline double lcos(double x){
    return (SinT[(int(x/pi2 * DIM)+DIM4)%DIM]);
  }

  inline double ljn(double x){
    return (BesT[int( (x+BESSEL_MAX)/BESSEL_MAX2 * DIMB)]);
  }

  inline double ldist(int s, int t){
    return (DisT[Grid.sDim*t+s]);
  }

  double lsdist(int q, int r, int s, int t){
    s=q-s;
    if (s<0) s=-s;
    t=r-t;
    if (t<0) t=-t;
    return (DisT[Grid.sDim*t+s]);
  }

  // returns e(-x)
  double lexp(double x){
    if (x>=EXP_MAX)return(0.0);
    return(ExpT[int(x/EXP_MAX*DIME)]);
  }
  
};

#endif /* lookup.h */
